package sun.security.util.math;

import java.math.BigInteger;

public interface IntegerModuloP
{
    public abstract IntegerFieldModuloP getField();
    
    public abstract BigInteger asBigInteger();
    
    public abstract ImmutableIntegerModuloP fixed();
    
    public abstract MutableIntegerModuloP mutable();
    
    public abstract ImmutableIntegerModuloP add(IntegerModuloP p0);
    
    public abstract ImmutableIntegerModuloP additiveInverse();
    
    public abstract ImmutableIntegerModuloP multiply(IntegerModuloP p0);
    
    default public byte[] addModPowerTwo(IntegerModuloP integerModuloP, int n) {
        byte[] array = new byte[n];
        this.addModPowerTwo(integerModuloP, array);
        return array;
    }
    
    public abstract void addModPowerTwo(IntegerModuloP p0, byte[] p1);
    
    default public byte[] asByteArray(int n) {
        byte[] array = new byte[n];
        this.asByteArray(array);
        return array;
    }
    
    public abstract void asByteArray(byte[] p0);
    
    default public ImmutableIntegerModuloP multiplicativeInverse() {
        return this.pow(this.getField().getSize().subtract(BigInteger.valueOf(2L)));
    }
    
    default public ImmutableIntegerModuloP subtract(IntegerModuloP integerModuloP) {
        return this.add(integerModuloP.additiveInverse());
    }
    
    default public ImmutableIntegerModuloP square() {
        return this.multiply(this);
    }
    
    default public ImmutableIntegerModuloP pow(BigInteger bigInteger) {
        MutableIntegerModuloP mutable = this.getField().get1().mutable();
        MutableIntegerModuloP mutable2 = this.mutable();
        for (int bitLength = bigInteger.bitLength(), i = 0; i < bitLength; ++i) {
            if (bigInteger.testBit(i)) {
                mutable.setProduct(mutable2);
            }
            mutable2.setSquare();
        }
        return mutable.fixed();
    }
}
